/*
	TaltOS Definitions
	on the Philips Chip LPC2106

	General Public Licence
	By Peter Eordegh
*/



//Component table pointer
#define TALTOS_COMPONENT_TABLE_PTR (0x0001C000)	//Sector 14 (sector 15 is the remapped bootloader)
#define TALTOS_COMPONENT_TABLE_SIZE (0x2000)	//This is a full sector size

//LPC2106 sepcific flash memory parameters
#define LPC2106_FLASH_MEMORY_SIZE (0x20000)		//All the flash memory size
#define LPC2106_SECTOR_SIZE (0x2000)			//Sector size 8192 bytes
#define LPC2106_MINDATA (0x200)					//1/16 part of the sector can be written 512 bytes
#define LPC2106_RAM_TOP_PTR (0x4000FFE0)

//Flash operation buffer
#define TALTOS_FLASH_OP_BUFFER_SIZE (0x2000)	//This is a full sector size
#define TALTOS_FLASH_OP_BUFFER_PTR (LPC2106_RAM_TOP_PTR-TALTOS_FLASH_OP_BUFFER_SIZE)

//Input data buffer
#define TALTOS_INPUT_BUFFER_SIZE (0x800)		//This is a half of a sector size
#define TALTOS_INPUT_BUFFER_PTR (TALTOS_FLASH_OP_BUFFER_PTR-TALTOS_INPUT_BUFFER_SIZE)

//Output data buffer
#define TALTOS_OUTPUT_BUFFER_SIZE (0x800)		//This is a half of a sector size
#define TALTOS_OUTPUT_BUFFER_PTR (TALTOS_INPUT_BUFFER_PTR-TALTOS_OUTPUT_BUFFER_SIZE)

#define TALTOS_ENV_PTR (TALTOS_OUTPUT_BUFFER_PTR-sizeof(TaltOS_env))

#define	TALTOS_TASK_LIST_SIZE (10)
#define TALTOS_TASK_LIST_PTR (TALTOS_ENV_PTR-(TALTOS_OUTPUT_BUFFER_PTR*sizeof(TaltOS_task)))


//TaltOS IDs
#define TALTOS_ID1 (0x0000014E)
#define TALTOS_ID2 (0x0000014E)

//TaltOS Functions (with some lpc2106 specific funtions for the vectored interrupt setup)
#define TALTOS_INIT					(0x00000000)
#define LPC2106_UART_HANDLER		(0x00000001)
#define LPC2106_TIMER_HANDLER		(0x00000002)
#define LPC2106_DEFAULT_HANDLER		(0x00000003)


#define TALTOS_ERASED (0x00000000)
#define TALTOS_EMPTY (0xFFFFFFFF)
#define TALTOS_COMPID_END (0x4E)

//The characters to identify if a component has arrived

//Endig Characters
#define TALTOS_COMP_END_1 ('*')
#define TALTOS_COMP_END_2 ('c')
#define TALTOS_COMP_END_3 ('m')
#define TALTOS_COMP_END_4 ('p')
#define TALTOS_COMP_END_5 ('*')

//Debug Macro

#define DEBUG(X) REG8(UART0_THR) = X;
