/*
	Bootloader for the TaltOS
	on the Philips Chip LPC2106

	General Public Licence
	By Peter Eordegh
*/


#include "lpc2106_init.h"
#include "lpc2106_memorymap.h"
#include "TaltOS_bootloader_defs.h"

short buffer_pointer;
short what;

typedef void (*IAP) (unsigned long[], unsigned long[]);

typedef struct
{
	unsigned long ID1;
	unsigned long ID2;
	unsigned long* address;
	unsigned long size;
} TaltOS_reg_entity;

long copy_to_flash_512(unsigned long to_address, unsigned long buf[], long data_size);

//Clear the input buffer
void clear_input_buffer()
{
	for(unsigned int i = 0;i <(TALTOS_INPUT_BUFFER_SIZE/sizeof(unsigned long));i++)
	{
		(*((unsigned long*)TALTOS_INPUT_BUFFER_PTR+i)) = (unsigned long)TALTOS_EMPTY;
	}
	buffer_pointer = 0;
}

//Clear the flash operation buffer
void clear_flash_op_buffer()
{
	for(unsigned int i = 0;i<(TALTOS_FLASH_OP_BUFFER_SIZE/sizeof(unsigned long));i++)
	{
		(*((unsigned long*)TALTOS_FLASH_OP_BUFFER_PTR+i)) = (unsigned long)TALTOS_EMPTY;
	}
}

unsigned int is_garbage_reg(char* test_array)
{
	if (test_array[0]==0x4E && test_array[4]==TALTOS_COMPID_END) {return 0;}
	else return 1;
}


//Remove all TaltOS registry entities
void remove_TaltOS()
{

	TaltOS_reg_entity *TaltOS_reg_flash;
	TaltOS_reg_entity *TaltOS_reg_buffer;
	unsigned int i;

	//Check the component table, if it is a TaltOS, write the complement if it's ID to the buffer
	for(i = 0; i < (TALTOS_FLASH_OP_BUFFER_SIZE/sizeof(TaltOS_reg_entity)); i++)
	{
		TaltOS_reg_flash = (TaltOS_reg_entity*)(TALTOS_COMPONENT_TABLE_PTR)+i;
		if ((TaltOS_reg_flash->ID1 == TALTOS_ID1)&&(TaltOS_reg_flash->ID2 == TALTOS_ID2))
		{
			TaltOS_reg_buffer = (TaltOS_reg_entity*)(TALTOS_FLASH_OP_BUFFER_PTR)+i;
			TaltOS_reg_buffer->ID1=~TALTOS_ID1;
			TaltOS_reg_buffer->ID2=~TALTOS_ID2;
			// We don't erase the address and size, for later garbage collection
		}
		else if ((TaltOS_reg_flash->ID1 != TALTOS_EMPTY)&&(TaltOS_reg_flash->ID2 != TALTOS_EMPTY))
		{
			if (is_garbage_reg((char*)TaltOS_reg_flash))
			{
				TaltOS_reg_buffer = (TaltOS_reg_entity*)(TALTOS_FLASH_OP_BUFFER_PTR)+i;
				TaltOS_reg_buffer->ID1=~TaltOS_reg_flash->ID1;
				TaltOS_reg_buffer->ID2=~TaltOS_reg_flash->ID1;
			}
		}
	}

	copy_to_flash_512(TALTOS_COMPONENT_TABLE_PTR,(unsigned long*)TALTOS_FLASH_OP_BUFFER_PTR,16);

	clear_flash_op_buffer();

	what=0;
}

//Run the TaltOS
void run_TaltOS()
{
	unsigned int i;
	bool haverun=false;
	TaltOS_reg_entity *TaltOS_reg;
	for(i=0; (i<TALTOS_COMPONENT_TABLE_SIZE/sizeof(TaltOS_reg_entity))&&(!haverun); i++)
	{
		TaltOS_reg=(TaltOS_reg_entity*)TALTOS_COMPONENT_TABLE_PTR+i;
		if ((TaltOS_reg->ID1==TALTOS_ID1)&&(TaltOS_reg->ID2==TALTOS_ID2))
		{
			unsigned long* rel_ptr = (unsigned long*)(TaltOS_reg->address);
			void (*TaltOS)() =(void(*)())(TaltOS_reg->address+(*rel_ptr)/4);
			TaltOS();
			haverun=true;
		}
	}
	if (!haverun){REG8(UART0_THR) = 'N';} // N as Not Found

	clear_input_buffer();

	what=0;
}

void copy_and_run_TaltOS()
{
	unsigned long* empty_tester;
	bool address_found=false;
	unsigned long i,j;
	TaltOS_reg_entity TaltOS;
	TaltOS_reg_entity *TaltOS_reg_flash;
	TaltOS_reg_entity *TaltOS_reg_buffer;
	TaltOS.ID1 = TALTOS_ID1;
	TaltOS.ID2 = TALTOS_ID2;
	//Get the size of the buffer to be written
	TaltOS.size = (buffer_pointer/LPC2106_MINDATA)+1;

	//Search for the first empty place of the TaltOS
	for(i=0; (i<LPC2106_FLASH_MEMORY_SIZE/LPC2106_MINDATA) && (!address_found); i++)
	{
		address_found=true;
		// Check if the full size is fit to this place
		for(j=0; (j<TaltOS.size)&&(address_found); j++)
		{
			empty_tester=(unsigned long*)((i+j)*LPC2106_MINDATA);
			if ((*empty_tester)!=TALTOS_EMPTY)
			{
				address_found=false;
			}
		}
	}

	if (address_found)
	{
		TaltOS.address = (unsigned long*)((i-1)*LPC2106_MINDATA);

		//Copy the buffer by LPC2106_MINDATA size blocks
		for(j=0; j<TaltOS.size; j++)
		{
			copy_to_flash_512(	(unsigned long)TaltOS.address+j*LPC2106_MINDATA,
								(unsigned long*)(TALTOS_INPUT_BUFFER_PTR+j*LPC2106_MINDATA),
								1);
		}

		clear_input_buffer();

		//Register the TaltOS if possible
		address_found=false;
		for(i = 0; (i < (TALTOS_FLASH_OP_BUFFER_SIZE/sizeof(TaltOS_reg_entity))) && (!address_found); i++)
		{
			TaltOS_reg_flash = (TaltOS_reg_entity*)(TALTOS_COMPONENT_TABLE_PTR)+i;
			if ((!address_found)&&(TaltOS_reg_flash->ID1 == TALTOS_EMPTY)&&(TaltOS_reg_flash->ID2 == TALTOS_EMPTY))
			{
				TaltOS_reg_buffer = (TaltOS_reg_entity*)(TALTOS_FLASH_OP_BUFFER_PTR)+i;
				TaltOS_reg_buffer->ID1=TaltOS.ID1;
				TaltOS_reg_buffer->ID2=TaltOS.ID2;
				TaltOS_reg_buffer->address=TaltOS.address;
				TaltOS_reg_buffer->size=TaltOS.size;
				address_found=true;
			}
		}

		//Copy the full component table back to the flash
		copy_to_flash_512(TALTOS_COMPONENT_TABLE_PTR,(unsigned long*)TALTOS_FLASH_OP_BUFFER_PTR,16);

		clear_flash_op_buffer();

		//Run the TaltOS
		what = 1;
	}
	else
	{
		REG8(UART0_THR) = 'F';		//F as Fail
		what = 0;
	}
}


//Copy 512 bytes to the Flash adress given
long copy_to_flash_512(unsigned long to_address, unsigned long buf[], long data_size)
{
	unsigned long command[5];								//Command pointer for IAP
	unsigned long result[2];								//Result pointer for IAP
	unsigned long vicmask = REG32(VICIntEnable);			//To save IT mas
	IAP iap_entry;											//The function pointer
	unsigned long sector = to_address/LPC2106_SECTOR_SIZE;	//Calculate which sector we want to write
	REG32(VICIntEnClr) = 0xFFFFFFFF;						//Disable IT
	iap_entry=(IAP) IAP_LOCATION;							//Get IAP routine pointer

	command[0]=50;											//Prepare sector to write/erase
	command[1]=sector;										// from sector
	command[2]=sector;										// till sector
	iap_entry(command,result);								//launch IAP
	//	REG8(UART0_THR) = result[0]+'0';

	command[0]=51;											//Write sector
	command[1]=to_address-(to_address%512);					//Address on correct boundary (512);
	command[2]=(unsigned long)buf;							//Buffer start
	command[3]=LPC2106_MINDATA*data_size;					//Buffer size
	//Processor clock frequenci in kHz (External cristal:10 MHz M=6)
	command[4]=60000;
	iap_entry(command,result);								//launch IAP

	//	REG8(UART0_THR) = result[0]+'0';
	REG32(VICIntEnable) = vicmask;							//Enable IT
	return result[0];
}




//Clear order from the end of the buffer
void end_order()
{
	buffer_pointer -= 5;
	*((unsigned char*)(TALTOS_INPUT_BUFFER_PTR)+buffer_pointer) = 0xFF;
	*((unsigned char*)(TALTOS_INPUT_BUFFER_PTR)+buffer_pointer+1) = 0xFF;
	*((unsigned char*)(TALTOS_INPUT_BUFFER_PTR)+buffer_pointer+2) = 0xFF;
	*((unsigned char*)(TALTOS_INPUT_BUFFER_PTR)+buffer_pointer+3) = 0xFF;
	*((unsigned char*)(TALTOS_INPUT_BUFFER_PTR)+buffer_pointer+4) = 0xFF;
}

// Chack if there is an order
char order(void)
{
	if (buffer_pointer >= 5)
	{	//Must start and end with '*'
		if ((*((unsigned char*)(TALTOS_INPUT_BUFFER_PTR)+buffer_pointer-5)=='*') &&
			(*((unsigned char*)(TALTOS_INPUT_BUFFER_PTR)+buffer_pointer-1)=='*'))
		{
			if ((*((unsigned char*)(TALTOS_INPUT_BUFFER_PTR)+buffer_pointer-4)=='r') &&//*run* - Run the TaltOS
				(*((unsigned char*)(TALTOS_INPUT_BUFFER_PTR)+buffer_pointer-3)=='u') &&
				(*((unsigned char*)(TALTOS_INPUT_BUFFER_PTR)+buffer_pointer-2)=='n'))
				{end_order();return 1;}
			if ((*((unsigned char*)(TALTOS_INPUT_BUFFER_PTR)+buffer_pointer-4)=='c') &&//*cnr* - Copy TaltOS to flash and run it
				(*((unsigned char*)(TALTOS_INPUT_BUFFER_PTR)+buffer_pointer-3)=='n') &&
				(*((unsigned char*)(TALTOS_INPUT_BUFFER_PTR)+buffer_pointer-2)=='r'))
				{end_order();return 2;}
			if ((*((unsigned char*)(TALTOS_INPUT_BUFFER_PTR)+buffer_pointer-4)=='r') &&//*rmt* - Remove TaltOS from flash
				(*((unsigned char*)(TALTOS_INPUT_BUFFER_PTR)+buffer_pointer-3)=='m') &&
				(*((unsigned char*)(TALTOS_INPUT_BUFFER_PTR)+buffer_pointer-2)=='t'))
				{end_order();return 3;}
			return 0;

		}
		else {return 0;}
	}
	else {return 0;}
}

// Timer interrupt - disabeled
void timer0_irq_handler(void)
{
	REG8(UART0_THR) = 't';
	REG8(TIMER0_IR) = 0xFF;		// Clear the interrupt flag
}

//Default interrupt handler
void default_irq_handler(void)
{
	REG8(UART0_THR) = '@';
}

//UART interrupt handler
void uart0_irq_handler(void)
{
	switch (REG8(UART0_IIR) & 0x0E) //Mask with 1110
	{
		case (0x02) :				//In case of THRE interrupt (more often interrupt)
		{
			REG8(UART0_IER) = 0x01; //If we have nothing to write out
			break;
		}
		case (0x04) :				//In case od RDA interrupt (less often interrupt)
		{
			*((unsigned char*)(TALTOS_INPUT_BUFFER_PTR)+buffer_pointer++) = REG8(UART0_RBR);
			what=order();	//Check if there was an order or not
			if (!(REG8(UART0_IER) & 0x02)){REG8(UART0_IER) = 0x03;}
			break;
		}
	}
}

int main (void)
{
	what=0;
	lpc2106_run();		//Init chip
	REG8(UART0_THR) = 'G';		//Write 'G' for Go
	REG8(TIMER0_TCR) = 0x0;		//Disable Timer interrupt
	clear_input_buffer();				//Clear the input buffer
	clear_flash_op_buffer();			//Clear the input buffer
	lpc2106_vectored_interrupts_init((unsigned long)timer0_irq_handler,
										(unsigned long) uart0_irq_handler,
										(unsigned long) default_irq_handler); //Set interrupt handlers
	if (!(REG8(UART0_IER) & 0x02)){REG8(UART0_IER) = 0x03;}
	while(1)	//Loop for order test
	{
		switch (what)
		{
			case (0) :  break;
			case (1) :  run_TaltOS();break;
			case (2) :  copy_and_run_TaltOS();break;
			case (3) :  remove_TaltOS();break;
		}
	}
}
